@@ -45,12 +45,13 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <time.h>
 
-// for inet_aton
+// for inet_ntoa
 #define _GNU_SOURCE 1
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include "ipv4_util2.h"
 #include "initial_count.h"
 #include "percent_prob.h"
 #include "ext_exec.h"
@@ -123,18 +124,14 @@ static char *db_block_name = IPQBDB_DATABASE_NAME;
 static char *db_white_name = IPQBDB_WHITE_DATABASE_NAME;
 static char *db_descr_name = IPQBDB_DESCR_DATABASE_NAME;
 static char *reason_string = "0";
-static char *ip_address = "0.0.0.0";
+static char **ip_address;
 static int version_opt, help_opt, syslog_opt, cleanup_opt, verbose = -1;
 static int force_probability_opt, force_decay_opt, force_reason_opt;
 static int exec_connkill_opt;
 static struct poptOption opttab[] =
 {
-	{"db-block", 'b', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_block_name, 0,
-	"The database of blocked IPv4 addresses", "filename"},
-	{"db-white", 'w', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_white_name, 0,
-	"The whitelist database.", "filename"},
-	{"db-descr", 'd', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_descr_name, 0,
-	"The reason description table.", "filename"},
+	{"ip-addr", 'i', POPT_ARG_ARGV, &ip_address, 0,
+	"An address to ban, repeatable", "ip_addr"},
 	{"reason", 'r', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &reason_string, 0,
 	"The reason string (or its record number) for the block record", "int-or-string"},
 	{"force-reason", '\0', POPT_ARG_NONE, &force_reason_opt, 0,
@@ -147,28 +144,32 @@ static struct poptOption opttab[] =
 	"Set initial probability count", "integer"},
 	{"force-probability", '\0', POPT_ARG_NONE, &force_probability_opt, 0,
 	"Change the probability even if the current value is higher", NULL},
-	{"ip-addr", 'i', POPT_ARG_STRING, &ip_address, 0,
-	"The address to ban, if any", "ip_addr"},
 	{"exec-connkill", 'e', POPT_ARG_NONE, &exec_connkill_opt, 0,
 	"Execute connection kill if probability reaches 100%", NULL},
 	{"verbose", 'v', POPT_ARG_INT|POPT_ARGFLAG_OPTIONAL, &verbose, 0,
 	"Be verbose", "level"},
 	{"log-syslog", 'l', POPT_ARG_NONE, &syslog_opt, 0,
 	"Log to syslog rather than std I/O", NULL},
-	{"db-cleanup", '\0', POPT_ARG_NONE, &cleanup_opt, 0,
-	"On exit cleanup environment (__db.00? files) if not still busy", NULL},
 	{"version", 'V', POPT_ARG_NONE, &version_opt, 0,
 	"Print version number and exit", NULL},
 	{"help", 'h', POPT_ARG_NONE, &help_opt, 0,
 	"This help.", NULL},
+	{"db-block", 'b', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_block_name, 0,
+	"The database of blocked IPv4 addresses", "filename"},
+	{"db-white", 'w', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_white_name, 0,
+	"The whitelist database.", "filename"},
+	{"db-descr", 'd', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_descr_name, 0,
+	"The reason description table.", "filename"},
+	{"db-cleanup", '\0', POPT_ARG_NONE, &cleanup_opt, 0,
+	"On exit cleanup environment (__db.00? files) if not still busy", NULL},
 	POPT_TABLEEND
 };
 
-static uint32_t get_ip_address(uint32_t* ip_addr)
+static int get_ip_address(char *ip_address, uint32_t* ip_addr)
 {
 	struct in_addr in;
-	int rtc = inet_aton(ip_address, &in);
-	if (rtc == 0) // not valid
+	int rtc = my_inet_pton(ip_address, &in);
+	if (rtc == 0 || in.s_addr == INADDR_ANY) // 0 is not valid
 	{
 		report_error(&ap, LOG_ERR, "invalid ip_address %s\n", ip_address);
 		return 1;
@@ -178,6 +179,30 @@ static uint32_t get_ip_address(uint32_t* ip_addr)
 	return 0;
 }
 
+static int get_all_ip_address(uint32_t **ip_addr)
+{
+	size_t count = 0;
+	char **a = ip_address;
+	while (*a++)
+		++count;
+
+	if (count == 0)
+	{
+		*ip_addr = NULL;
+		return 0;
+	}
+
+	uint32_t *addr = *ip_addr = calloc(count + 1, sizeof(uint32_t));
+	if (addr == NULL)
+		return -1;
+
+	for (a = ip_address; *a; ++a, ++addr)
+		if (get_ip_address(*a, addr))
+			return -1;
+
+	return 0;
+}
+
 static int display_check_whitelist(DB *db, uint32_t ip_addr, double *decay)
 {
 	int const rtc = check_whitelist(db, ip_addr, decay);
@@ -329,11 +354,21 @@ write_block_display(DB *db, uint32_t ip_addr, double decay, int reason_id, DB *d
 	return probability >= RAND_MAX;
 }
 
+static void run_exec_ip(connkill_cmd *cmd, uint32_t ip_addr)
+{
+	char *ip_address;
+	struct in_addr in;
+	memcpy(&in, &ip_addr, sizeof in);
+
+	ip_address = strdup(inet_ntoa(in));
+	run_exec(cmd, ip_address, &ap);
+}
+
 int main(int argc, char const *argv[])
 {
 	static const char optaliases[] = IPQBDB_OPTION_FILE;
 	int rtc = 0, errs = 0;
-	uint32_t ip_addr;
+	uint32_t *ip_addr = NULL;
 
 	poptContext opt = poptGetContext(err_prefix, argc, argv, opttab, 0);
 
@@ -389,7 +424,7 @@ int main(int argc, char const *argv[])
 			errs = 1;
 		}
 
-		if (get_ip_address(&ip_addr))
+		if (get_all_ip_address(&ip_addr))
 			errs = 3;
 
 		if (initial_decay < 0.0)
@@ -451,8 +486,7 @@ int main(int argc, char const *argv[])
 		{
 			DB_ENV *db_env = NULL;
 			DB *db_block = NULL, *db_white = NULL, *db_de1 = NULL, *db_de2 = NULL;
-			double decay;
-			int exec_connkill = 0;
+			uint32_t exec_connkill = 0;
 
 			setsigs();
 
@@ -470,15 +504,23 @@ int main(int argc, char const *argv[])
 					&ap, db_de1, db_de2, verbose);
 
 			if (rtc == 0 && ip_addr && caught_signal == 0)
-				rtc = display_check_whitelist(db_white, ip_addr, &decay);
-
-			if (rtc == 0 && ip_addr && caught_signal == 0 && decay > 0.0)
 			{
-				rtc = write_block_display(db_block, ip_addr, decay, reason_id, db_de1);
-				if (rtc > 0)
+				double decay;
+				uint32_t *next_ip = ip_addr;
+				while (rtc == 0 && *next_ip && caught_signal == 0)
 				{
-					exec_connkill = 1;
-					rtc = 0;
+					rtc = display_check_whitelist(db_white, *next_ip, &decay);
+
+					if (rtc == 0 && caught_signal == 0 && decay > 0.0)
+					{
+						int a_rc = write_block_display(
+							db_block, *next_ip, decay, reason_id, db_de1);
+						if (a_rc > 0)
+							exec_connkill = *next_ip;
+						else if (a_rc != 0)
+							rtc = -1; // bail out on error
+					}
+					++next_ip;
 				}
 			}
 
@@ -496,11 +538,13 @@ int main(int argc, char const *argv[])
 			{
 				connkill_cmd *cmd = read_connkill_cmd(&ap);
 				if (cmd)
-					run_exec(cmd, ip_address, &ap);
-				rtc = 1;
+					run_exec_ip(cmd, exec_connkill);
+
+				rtc = 2; // either exec or cmd failed
 			}
 		}
 	}
 
 	return rtc;
 }
+
