@@ -31,6 +31,7 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <db.h>
 #include "config_names.h"
 #include "dbstruct.h"
+#define INITIAL_COUNT_H_ONE
 #include "initial_count.h"
 
 static const char debug_status[] =
@@ -71,7 +72,6 @@ int main()
 		"RECENT_DROP_TIME %d\n"
 		"UPDATE_TICK_TIME %d\n"
 		"IPQBDB_INITIAL_DECAY %g\n"
-		"IPQBDB_DECAY_ADDEND_ON_THRESHOLD %g\n"
 		"IPQBDB_PROBABILITY_THRESHOLD %d (RAND_MAX %jd)\n"
 		"IPQBDB_NEVER_DECAY %f\n#\n"
 		"IPQBDB_DEBUG %s\n"
@@ -79,7 +79,6 @@ int main()
 		RECENT_DROP_TIME,
 		UPDATE_TICK_TIME,
 		IPQBDB_INITIAL_DECAY,
-		IPQBDB_DECAY_ADDEND_ON_THRESHOLD,
 		IPQBDB_PROBABILITY_THRESHOLD, (intmax_t)RAND_MAX,
 		IPQBDB_NEVER_DECAY,
 		debug_status,
