@@ -56,9 +56,10 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include <arpa/inet.h>
 
 #include "ipv4_util.h"
-#include "initial_count.h"
 #include "percent_prob.h"
 #include "rehabilitated_prob.h"
+#define INITIAL_COUNT_H_BOTH
+#include "initial_count.h"
 
 #include <pcre.h> // for grepping the reason table
 #include <assert.h>
@@ -102,26 +103,44 @@ time_string(time_t datum, time_t now, char *dest)
 
 /*
 * adaptive deletion
+* Problem: Delete records older than a given date, but only if this is needed
+* to keep the database within the given number of records.
 *
-* Read matching block.db records and write an histogram of updated dates. It
-* is quicker than sorting the records in order to delete only the oldest ones,
-* but it is also less precise.
+* If records were sorted by their increasing age, the problem would be solved
+* by deleting all records after either the given max size or the given time,
+* whichever comes last.  Instead, we read the matching block.db records in
+* their natural order, and build an histogram of the number of records in each
+* period of time.  From the histogram we determine the boundary date such that
+* the problem is solved by deleting the records older than that.  This implies
+* reading block.db for a second time, but is still quicker than sorting it.  It
+* is also less precise.
 *
-* If the user has given a period P > 0, we can be more precise in that order of
-* magnitude by mapping linearly times less than P onto the first N entries,
-* that is index(t) = N*t/P; and thereafter using logarithms, that is index(t) =
-* log(k*t)/log(f) with constants factor f and scale k. For continuity, we need
-* log(k*P)/log(f) = N; that is, log(f) = log(k*P)/N. For smoothness, we need
-* the derivatives of index(t) to match at t=P, i.e. N/(P*log(k*P)) = N/P; that
-* implies log(k*P) = 1, or k = e/P, if using natural logarithms. Thus,
-* log(f) = 1/N, and the mapping is index(t) = log(k*t)*N for t > P.
+* If the user has given a period P > 0, we can be more precise around that
+* order of magnitude by mapping linearly times less than P onto the first N
+* entries, that is index(t) = N*t/P; and using logarithms thereafter, that is
+* index(t) = log(k*t)/log(f), where the factor f and the scale k are constants
+* that we determine like so:  For continuity, we need log(k*P)/log(f) = N; that
+* is, log(f) = log(k*P)/N.  For smoothness, we need the derivatives of index(t)
+* to match at t=P, i.e. N/(P*log(k*P)) = N/P; that implies log(k*P) = 1, or
+* k = e/P, if using natural logarithms.  Thus, we find log(f) = 1/N, and the
+* mapping to get the index of a histogram bar from the age is
 *
-* The inverse mapping is time(i) = exp(i/N)/k.
+*             index(t) = log(k*t)*N   for t > P
+*                        N*t/P        for t <= P
 *
-* To ensure time(L*N) > M, we need exp(L-1) > M/P; that is L > 1+log(M/P). So
+* The inverse mapping is
+*
+*             time(i) = exp(i/N)/k    for i > N
+*                       P*i/N         for i <= N
+*
+* We need to determine the size of the histogram and the value of N.  Let M be
+* the maximum age that we want to map linearly, and L the time span covered by
+* a linearly mapped bar of the histogram.
+*
+* To ensure time(L*N) > M, we need exp(L-1) > M/P; that is L > 1+log(M/P).  So
 * we can set N to a reasonable value by choosing, say, M = 4 years and an array
-* of, say, 256; then we calculate N as floor(256/(1+log(M/P))). With these
-* settings we get the following, for different values of the period:
+* of, say, 256 entries; then we calculate N as floor(256/(1+log(M/P))).  With
+* these settings we get the following, for different values of the period:
 
 period     N    time(256)/M
 1 hour    22    1.18700507875353039495
@@ -132,6 +151,11 @@ period     N    time(256)/M
 30 days   52    1.03810599376929316910
 6 month   83    1.00490781176609466834
 1 year   107    1.00624844395160134207
+*
+* N is called "mesh" below.  The period is determined as 1/4 of the min_time,
+* that is when selecting records that must be at least that old.  The sense of
+* max_time is rather different: it is not related with the maximum number of
+* records, so we just select all records that are not older than that.
 */
 #define HISLOG_SIZE 256
 typedef struct histog
@@ -399,6 +423,7 @@ static char *db_block_name = IPQBDB_DATABASE_NAME;
 static char *db_descr_name = IPQBDB_DESCR_DATABASE_NAME;
 static char *ip_address = "0.0.0.0/0";
 static char *min_time = "0";
+static char *max_time;
 static char *max_probability = "100%";
 static char *min_probability = "0%";
 static char *reason_string = "";
@@ -410,22 +435,20 @@ static int verbose = -1;
 static int list_opt, raw_opt, del_opt, ban_opt, stat_opt;
 static struct poptOption opttab[] =
 {
-	{"db-block", 'b', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_block_name, 0,
-	"The database of blocked IPv4 addresses", "filename"},
-	{"db-descr", 'd', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_descr_name, 0,
-	"The reason description table.", "filename"},
 	{"only-blocked", '\0', POPT_ARG_NONE, &only_blocked, 0,
 	"Only select records with blocked packets > 0", NULL},
 	{"only-never-blocked", '\0', POPT_ARG_NONE, &non_blocked, 0,
 	"Only select records with blocked packets = 0", NULL},
+	{"max-records", 'M', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &max_records_opt, 0,
+	"Limit selection to older records that exceed the given number", "int[K|M|G]"},
 	{"min-time", 't', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &min_time, 0,
-	"The minimum time elapsed since record was last updated", "int[D|H|M|S]"},
+	"The minimum time elapsed since record was last updated or blocked", "int[D|H|M|S]"},
+	{"max-time", '\0', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &max_time, 0,
+	"Select records that have been blocked more recently than this", "int[D|H|M|S]"},
 	{"min-probability", '\0', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &min_probability, 0,
 	"Select records with probability higher than given(*)", "int[%]"},
 	{"max-probability", '\0', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &max_probability, 0,
 	"Select records with probability lower than given(*)", "int[%]"},
-	{"max-records", 'M', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &max_records_opt, 0,
-	"Limit selection to older records that exceed the given number", "int[K|M|G]"},
 	{"ip-addr", 'i', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &ip_address, 0,
 	"The single address or range to delete or list, if any", "ip[-last|/cidr]"},
 	{"reason", 'r', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &reason_string, 0,
@@ -444,15 +467,21 @@ static struct poptOption opttab[] =
 	"Be verbose", "level"},
 	{"log-syslog", 'l', POPT_ARG_NONE, &syslog_opt, 0,
 	"Log to syslog rather than std I/O", NULL},
-	{"db-cleanup", '\0', POPT_ARG_NONE, &cleanup_opt, 0,
-	"On exit cleanup environment (__db.00? files) if not still busy", NULL},
 	{"version", 'V', POPT_ARG_NONE, &version_opt, 0,
 	"Print version number and exit", NULL},
 	{"help", 'h', POPT_ARG_NONE, &help_opt, 0,
 	"This help.", NULL},
+	{"db-block", 'b', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_block_name, 0,
+	"The database of blocked IPv4 addresses", "filename"},
+	{"db-descr", 'd', POPT_ARG_STRING|POPT_ARGFLAG_SHOW_DEFAULT, &db_descr_name, 0,
+	"The reason description table.", "filename"},
+	{"db-cleanup", '\0', POPT_ARG_NONE, &cleanup_opt, 0,
+	"On exit cleanup environment (__db.00? files) if not still busy", NULL},
 	POPT_TABLEEND
 };
 
+static time_t time_invoked;
+
 static int *reason_id, count_reason_id, alloc_reason_id;
 
 static int add_reason_id(int ndx)
@@ -597,11 +626,11 @@ static int all_reason_id(DB *db1, DB *db2)
 	return rtc;
 }
 
-static int get_seconds(time_t *secs)
+static int get_seconds(time_t *secs, char *opt_time)
 {
 	errno = 0;
 	char *t = NULL;
-	unsigned long long l = strtoull(min_time, &t, 10);
+	unsigned long long l = strtoull(opt_time, &t, 10);
 	if (t && *t && errno == 0)
 	{
 		switch (toupper(*(unsigned char*)t))
@@ -622,7 +651,7 @@ static int get_seconds(time_t *secs)
 
 	if (t == NULL || *t != 0 || (time_t)l < 0)
 	{
-		report_error(&ap, LOG_ERR, "invalid time %s\n", min_time);
+		report_error(&ap, LOG_ERR, "invalid time %s\n", opt_time);
 		return 1;
 	}
 
@@ -705,7 +734,7 @@ static int get_ip_address(ip4_range *ip)
 }
 
 static int
-do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
+do_del(DB *db, DB*db_d, ip4_range* ip, time_t *secs, int *prob, histog *his)
 /*
 * If his != NULL, compute the histogram of records exceeding the given min-time
 * and perform no other operation.
@@ -717,12 +746,10 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 	uint32_t cur_key = ip->first;
 	int rtc, rtc2;
 	unsigned total_selected = 0, total_del = 0, total_delfail = 0, total_recs = 0;
-	time_t const now = time(NULL), last_time = now - secs;
-
-	int max_count;
-	if (ban_opt)
-		max_count = max_initial_count();
+	time_t const now = time_invoked, last_time = now - secs[0],
+		first_time = now - secs[1]; // 0 by default, or implies only-blocked
 
+	int max_count = ban_opt? max_initial_count(): 0;
 
 	char descr[IPQBDB_DESCR_MAX_LENGTH];
 
@@ -805,15 +832,31 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 			total_recs += 1;
 
 			/* test for specific selection criteria */
-			int const selected = (secs == 0 ||
+			int selected = (secs[0] == 0 ||
 					(ip_data->last_update <= last_time &&
-					ip_data->last_block <= last_time)) &&
-				ip_data->probability >= prob[0] &&
-				ip_data->probability <= prob[1] &&
+						ip_data->last_block <= last_time)) &&
+				ip_data->last_block >= first_time && // ==> was blocked || first==0
 				(only_blocked == 0 || ip_data->block_cnt > 0) &&
 				(non_blocked == 0 || ip_data->block_cnt == 0) &&
 				(reason_id == NULL || have_reason_id(ip_data->reason_id));
 
+			time_t delta = now - ip_data->last_update;
+			int adjusted_prob = ip_data->probability;
+
+			if (selected)
+			{
+				if (adjusted_prob > 0 &&
+					UPDATE_TICK_TIME < delta &&
+					ip_data->decay > 0.0)
+						adjusted_prob = 
+							rehabilitated_prob(adjusted_prob, delta, ip_data->decay);
+
+				// adjusted prob should never be negative, but just in case...
+				if (!((adjusted_prob >= prob[0] || prob[0] == 0) &&
+					adjusted_prob <= prob[1]))
+						selected = 0;
+			}
+
 			if (selected)
 			{
 				if (his)
@@ -847,6 +890,8 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 					}
 				}
 
+				char ntopbuf[INET_ADDRSTRLEN];
+
 				if (raw_opt)
 				{
 					if (verbose && !total_selected)
@@ -855,7 +900,8 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 							"UPDATED", "DECAY", "THRESHOLD", "CAUGHT", "DESCRIPTION");
 
 					printf("%15s %11d %11d %11d %11d %11d %11g %11d %11d %11d\n",
-						inet_ntoa(*(struct in_addr*)&cur_key),
+						inet_ntop(AF_INET, (struct in_addr*)&cur_key,
+							ntopbuf, sizeof ntopbuf),
 						(int)ip_data->created,
 						ip_data->probability,
 						(int)ip_data->last_block,
@@ -872,16 +918,6 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 						buf2[TIME_STRING_BUFSIZE],
 						buf3[TIME_STRING_BUFSIZE];
 
-					time_t delta = now - ip_data->last_update;
-					int reported_prob = ip_data->probability;
-					
-					if (reported_prob > 0 &&
-						UPDATE_TICK_TIME < delta &&
-						ip_data->decay > 0.0)
-							reported_prob = 
-								rehabilitated_prob(reported_prob,
-									delta, ip_data->decay);
-
 					if (verbose && list_opt && !total_selected)
 						printf("%15s %8s %7s %8s %10s %8s %10s %10s %10s %s\n",
 							"IP", "CREATED", "PROB.", "BLOCKED", "PACKETS",
@@ -891,9 +927,10 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 					if (list_opt)
 					{
 						printf("%15s %8s %6.2f%% %8s %10d %8s %10g %10d %10d %s\n",
-							inet_ntoa(*(struct in_addr*)&cur_key),
+							inet_ntop(AF_INET, (struct in_addr*)&cur_key,
+								ntopbuf, sizeof ntopbuf),
 							time_string(ip_data->created, now, buf1),
-							percent_prob(reported_prob),
+							percent_prob(adjusted_prob),
 							time_string(ip_data->last_block, now, buf2),
 							ip_data->block_cnt,
 							time_string(ip_data->last_update, now, buf3),
@@ -912,8 +949,9 @@ do_del(DB *db, DB*db_d, ip4_range* ip, time_t secs, int *prob, histog *his)
 					if (ban_opt)
 						printf("ibd-ban --ip-addr=%s --initial-count=%d"
 							" --initial-decay=%g --reason='%s'\n",
-							inet_ntoa(*(struct in_addr*)&cur_key),
-							guess_initial_count(reported_prob, max_count),
+							inet_ntop(AF_INET, (struct in_addr*)&cur_key,
+								ntopbuf, sizeof ntopbuf),
+							guess_initial_count(adjusted_prob, max_count),
 							ip_data->decay,
 							descr);
 				}
@@ -1047,10 +1085,13 @@ static int do_stat(DB *db)
 int main(int argc, char const *argv[])
 {
 	static const char optaliases[] = IPQBDB_OPTION_FILE;
+	static char max_max_time[32];
 	int rtc = 0, errs = 0, prob[2] = {0, 0};
-	time_t secs = 0;
+	time_t secs[2] = {0, time(&time_invoked)};
 	ip4_range ip;
 
+	sprintf(max_max_time, "%ldS", (long)secs[1]);
+	max_time = max_max_time;
 	poptContext opt = poptGetContext(err_prefix, argc, argv, opttab, 0);
 
 	if (access(optaliases, F_OK) == 0 &&
@@ -1087,13 +1128,10 @@ int main(int argc, char const *argv[])
 		{
 			poptPrintHelp(opt, stdout, 0);
 			fputs("\n"
-"[*] Note that the min and max probabilities are compared against the values\n"
-"actually stored in the database, without accounting for rehabilitation. Thus\n"
-"the meaning of the selection varies for records with decay values (seconds)\n"
-"much smaller than the time elapsed since the last update: because ibd-judge\n"
-"only updates records when it sees a packet, the selection obtained with these\n"
-"options reflects rather the history of the relevant IPs than the action that\n"
-"would be taken if a new packet is seen from them.\n", stdout);
+"[*] Note that the min and max probabilities are compared accounting for\n"
+"rehabilitation, not the bare values stored into the database. Thus the values\n"
+"displayed in raw listing may differ from the values given for these options.\n",
+				stdout);
 			fputs_database_help();
 			errs = 2;
 		}
@@ -1123,12 +1161,22 @@ int main(int argc, char const *argv[])
 			}
 		}
 
-		if (get_ip_address(&ip) || get_seconds(&secs) ||
+		if (get_ip_address(&ip) ||
+			get_seconds(&secs[0], min_time) ||
+			get_seconds(&secs[1], max_time) ||
 			get_max_records() ||
 			get_probability(&prob[0], min_probability) ||
 			get_probability(&prob[1], max_probability))
 				errs = 3;
 
+		if (secs[0] >= secs[1] && errs != 3)
+		{
+			report_error(&ap, LOG_WARNING, "no record can be"
+				" at least as old as min-time=%s and at most as max-time=%s\n",
+				min_time, max_time);
+			errs = 1;
+		}
+
 		// popt sets verbose to 0 if no arg is given
 		verbose += 1;
 
@@ -1179,7 +1227,7 @@ int main(int argc, char const *argv[])
 			if (rtc == 0 && caught_signal == 0 && max_records > 0)
 			{
 				histog his;
-				if (init_histog(&his, secs) == 0)
+				if (init_histog(&his, secs[0]) == 0)
 				{
 					rtc = do_del(db_block, db_de1, &ip, secs, prob, &his);
 					if (rtc == 0)
@@ -1192,11 +1240,10 @@ int main(int argc, char const *argv[])
 									his.total_recs, max_records);
 							del_opt = 0;
 						}
-						else
-							if (his.total_recs - his.total_selected < max_records)
+						else if (his.total_recs - his.total_selected < max_records)
 						{
 							char const* err =
-								increase_secs(&his, max_records, secs);
+								increase_secs(&his, max_records, secs[0]);
 							if (err)
 								report_error(&ap, LOG_ERR,
 									"increase_secs: %s\n", err);
@@ -1210,8 +1257,8 @@ int main(int argc, char const *argv[])
 									{
 										char before[TIME_STRING_BUFSIZE],
 											after[TIME_STRING_BUFSIZE];
-										time_string(his.now - secs, his.now, before);
-										time_string(his.now - secs - his.increase,
+										time_string(his.now - secs[0], his.now, before);
+										time_string(his.now - secs[0] - his.increase,
 											his.now, after);
 										if (strcmp(before, after))
 											printf(" min age bumped by %jd: %s to %s!\n",
@@ -1224,7 +1271,7 @@ int main(int argc, char const *argv[])
 									else
 										printf(" min age did not vary!\n");
 								}
-								secs += his.increase;
+								secs[0] += his.increase; // may get >= secs[1]
 							}
 						}
 						else
@@ -1248,7 +1295,7 @@ int main(int argc, char const *argv[])
 				if (verbose)
 				{
 					printf("%s%s%s records in IP range %d.%d.%d.%d-%d.%d.%d.%d,\n"
-						"\t%s%smin age %jd secs,"
+						"\t%s%smin age %jd secs, max age %jd secs,"
 						" min prob %d=%.2f%%, max prob %d=%.2f%%.\n",
 						del_opt? "delete": "",
 						del_opt && (list_opt || raw_opt || ban_opt)? " and ": "",
@@ -1263,7 +1310,7 @@ int main(int argc, char const *argv[])
 						((unsigned char*)&ip.last)[3],
 						only_blocked? "only blocked, ": "",
 						non_blocked? "non blocked, ": "",
-						(intmax_t)secs,
+						(intmax_t)secs[0], (intmax_t)secs[1],
 						prob[0], percent_prob(prob[0]),
 						prob[1], percent_prob(prob[1]));
 				}
