@@ -55,7 +55,7 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <sys/wait.h>
 
-// for inet_aton
+// for inet_ntoa
 #define _GNU_SOURCE 1
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -65,6 +65,7 @@ along with Ipqbdb.  If not, see <http://www.gnu.org/licenses/>.
 #include "config_names.h"
 #include "dbstruct.h"
 
+#include "ipv4_util2.h"
 #include "initial_count.h"
 #include "percent_prob.h"
 #include "ext_exec.h"
@@ -1002,7 +1003,7 @@ static int get_ip_address(uint32_t* ip, char *dest, size_t dsize,
 
 	memcpy(dest, src, ssize);
 	dest[ssize] = 0;
-	int rtc = inet_aton(dest, &in);
+	int rtc = my_inet_pton(dest, &in);
 	if (rtc == 0) // not valid
 	{
 		report_error(&ap, LOG_WARNING,
@@ -1409,15 +1410,22 @@ int main(int argc, char const *argv[])
 			poptPrintHelp(opt, stdout, 0);
 			fputs_database_help();
 			fputs("\nNon-comment lines in the pcre-file are formatted as\n\n"
-			"\t/pcre/ N \"reason\" [initial-count] [filename] [decay]\n\n"
-			"where N is the integer for the captured pcre subexpression. "
-			"Slashes around\npcre can be replaced, quotes around the reason are "
-			"optional (can specify the\ncorresponding integer) and the optional "
-			"fields can be replaced by a hyphen (-).\nThe filename is the block "
-			"database, subject to prefix/suffix arrangenments.\nAll DB filenames"
-			" must end up in the same directory, which is also used for\nthe "
-			"socket.\n", stdout);
+			"\t/pcre/ I \"reason\" [initial-count] [filename] [decay]\n\n"
+			"where I is * for <HOST>, - for named subexpression, or the index of\n"
+			"the pcre subexpression. Slashes around pcre can be replaced, quotes\n"
+			"around reason are optional (can specify the corresponding integer).\n",
+				stdout);
 			fputs_initial_count_help();
+			fputs("\n"
+			"The filename is that of the block database, subject to prefix/suffix\n"
+			"arrangements.  All DB filenames must end up in the same directory,\n"
+			"which is also used for\nthe socket.\n"
+			"\n"
+			"The decay defines the halving period, in seconds, and overrides the\n"
+			"initial-decay (see option -t above) for each expression/reason.\n"
+			"\n"
+			"Any optional field can be replaced by a hyphen (-).\n",
+				stdout);
 			errs = 2;
 		}
 
